﻿;   BarcodeModule Version 1.0.0 Beta 18/02/2018
;
;   PureBasic 5.61 (Windows x64)
;
;   MIT License
;
;   Copyright 2018 Michael Suther
; 
;   Permission is hereby granted, free of charge, To any person obtaining a copy of this software And associated documentation files (the "Software"),
;   To deal in the Software without restriction, including without limitation the rights To use, copy, modify, merge, publish, distribute, sublicense,
;   And/Or sell copies of the Software, And To permit persons To whom the Software is furnished To do so, subject To the following conditions:
; 
;   The above copyright notice And this permission notice shall be included in all copies Or substantial portions of the Software.
; 
;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS Or IMPLIED, INCLUDING BUT Not LIMITED To THE WARRANTIES OF MERCHANTABILITY,
;   FITNESS For A PARTICULAR PURPOSE And NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS Or COPYRIGHT HOLDERS BE LIABLE For ANY CLAIM, DAMAGES Or OTHER LIABILITY,
;   WHETHER IN AN ACTION OF CONTRACT, TORT Or OTHERWISE, ARISING FROM, OUT OF Or IN CONNECTION With THE SOFTWARE Or THE USE Or OTHER DEALINGS IN THE SOFTWARE.


DeclareModule Common
  EnableExplicit
  Structure Barcode_Parameter
    Type.s                        ;Beispiel: "EAN8", "EAN13"
    Barcode.s                     ;Die Code Ziffern/Buchstaben. Beispiel EAN8: "72395677"
    Font.s                        ;Beispiel: "Arial". Die Größe wird automatisch angepasst. Wird kein Font angegeben, Wird der Code OHNE Ziffern gedruckt.
    Sequence.s                    ;Die Prozedur Generate_EAN8_Sequence liefert diese
    Width.i                       ;Die Barcode Breite in mm
    Height.i                      ;Die Barcode Höhe in mm
    PosX.i                        ;Position in mm
    PosY.i                        ;Position in mm
    Color0_RGB.s                  ;Farbe der hellen Code Linien. Beispiel: "255,255,255". Komma = Trennzeichen
    Color1_RGB.s                  ;Farbe der dunklen Code Linien. Beispiel: "0,0,0". Komma = Trennzeichen
    ColorDigits_RGB.s             ;Farbe der Code Ziffern. Beispiel: "0,0,0". Komma = Trennzeichen
  EndStructure
  
  Structure Print_Parameter
    Page_width.d                  ;Papierbreite in mm
    Page_height.d                 ;Papierhöhe in mm
    Left_edge.d                   ;Druckrand links in mm
    Right_edge.d                  ;Druckrand rechts in mm
    Top_edge.d                    ;Druckrand oben in mm
    Upper_edge.d                  ;Druckrand unten in mm
    Pages.i                       ;Anzahl zu druckender Seiten
    Print_Requester.b             ;#True = Es wird das Drucker Requester geöffnet. #False = Es wird der Standard Drucker verwendet.
  EndStructure
  
  Structure Text_Line
    Text.s                        ;Der zu druckende Text
    Font.s                        ;Beispiel: "Arial"
    Font_Size.i                   ;Beispiel: 12
    TextColor_RGB.s               ;Die Farbe der Textzeile
    Text_Style.i                  ;Beispiel: #PB_Font_Underline|#PB_Font_Italic
    Text_PosX.i                   ;Position der Textzeile in mm
    Text_PosY.i                   ;Position der Textzeile in mm
  EndStructure
  
EndDeclareModule
Module Common
  ;Nichts
EndModule

DeclareModule Barcode
  EnableExplicit
  UseModule Common
  
  ;Diese Funktion benötigt die 7 Ziffern (digits) des EAN8 Codes, als String.
  Declare Generate_EAN8_Checksum(digits.s)
  
  ;Diese Funktion benötigt die 12 Ziffern (digits) des EAN13 Codes, als String.
  Declare Generate_EAN13_Checksum(digits.s)
  
  ;Diese Funktion benötigt den kompletten EAN8 Code inkl. Prüfziffer, als String.
  Declare.s Generate_EAN8_Sequence(EAN8.s)
  
  ;Diese Funktion benötigt den kompletten EAN13 Code inkl. Prüfziffer, als String.
  Declare.s Generate_EAN13_Sequence(EAN13.s)
  
  ;Siehe Beispiele
  Declare Print_Barcode(List Barcode.Barcode_Parameter(), *Printer.Print_Parameter, List Text.Text_Line())
  
EndDeclareModule

Module Barcode
  
  Procedure Generate_EAN8_Checksum(digits.s)
    Protected.s digit
    Protected i, Dim digit_num(6), a = 0, Odd, Even, checksum
    ; Der String muss 7 Ziffern enthalten. Wenn nein, Abbruch
    If Len(digits.s) <> 7
      ProcedureReturn 10
    EndIf
    
    ;Den String digits.s in 7 Zahlen zerlegen
    For i = 0 To 13 Step 2
      digit.s = PeekS(@digits.s + i, 1, #PB_Unicode)
      If UCase(digit.s) = LCase(digit.s) ;Pruefen ob wirklich numerisch
        digit_num(a) = Val(digit.s)
        a = a + 1
      Else
        ProcedureReturn 11
      EndIf
    Next i
    ;Odd und Even errechnen
    Odd   = digit_num(6) + digit_num(4) + digit_num(2) + digit_num(0)
    Even  = digit_num(5) + digit_num(3) + digit_num(1)
    ;Prüfziffer berechnen
    checksum = (10 - (3 * Odd + Even) % 10) %10
    ProcedureReturn checksum
  EndProcedure
  
  Procedure Generate_EAN13_Checksum(digits.s)
    Protected.s digit
    Protected i, Dim digit_num(11), a, Odd, Even, checksum
    
    ; Der String muss 12 Ziffern enthalten. Wenn nein, Abbruch
    If Len(digits.s) <> 12
      ProcedureReturn 10
    EndIf
    
    ;Den String digits.s in 12 Zahlen zerlegen
    For i = 0 To 23 Step 2
      digit.s = PeekS(@digits.s + i, 1, #PB_Unicode)
      If UCase(digit.s) = LCase(digit.s) ;Pruefen ob wirklich numerisch
        digit_num(a) = Val(digit.s)
        a = a + 1
      Else
        ProcedureReturn 11
      EndIf
    Next i
    ;Odd und Even errechnen
    Odd   = digit_num(11) + digit_num(9) + digit_num(7) + digit_num(5) + digit_num(3) + digit_num(1)
    Even  = digit_num(10) + digit_num(8) + digit_num(6) + digit_num(4) + digit_num(2) + digit_num(0)
    ;Prüfziffer berechnen
    checksum = (10 - (3 * Odd + Even) % 10) %10
    ProcedureReturn checksum
  EndProcedure
  
  Procedure.s Generate_EAN8_Sequence(EAN8.s)  
    Protected.b i, a
    Protected.s part
    ;Wird die gesamte EAN8 Sequence enthalten
    Protected EAN8_Sequence.s  
    ;EAN8 Start, Center und Stop Sequencen
    Protected EAN8_Start.s   = "101"
    Protected EAN8_Center.s  = "01010" 
    Protected EAN8_Stop.s    = "101"   
    ;EAN8 Sequenzen part #1
    Protected Dim EAN8_Sequence_part1.s(9)
    EAN8_Sequence_part1(0) = "0001101"
    EAN8_Sequence_part1(1) = "0011001"
    EAN8_Sequence_part1(2) = "0010011"
    EAN8_Sequence_part1(3) = "0111101"
    EAN8_Sequence_part1(4) = "0100011"
    EAN8_Sequence_part1(5) = "0110001"
    EAN8_Sequence_part1(6) = "0101111"
    EAN8_Sequence_part1(7) = "0111011"
    EAN8_Sequence_part1(8) = "0110111"
    EAN8_Sequence_part1(9) = "0001011"
    ;EAN8 Sequenzen part #2
    Protected Dim EAN8_Sequence_part2.s(9)
    EAN8_Sequence_part2(0) = "1110010"
    EAN8_Sequence_part2(1) = "1100110"
    EAN8_Sequence_part2(2) = "1101100"
    EAN8_Sequence_part2(3) = "1000010"
    EAN8_Sequence_part2(4) = "1011100"
    EAN8_Sequence_part2(5) = "1001110"
    EAN8_Sequence_part2(6) = "1010000"
    EAN8_Sequence_part2(7) = "1000100"
    EAN8_Sequence_part2(8) = "1001000"
    EAN8_Sequence_part2(9) = "1110100"
    
    ; Der String muss 8 Ziffern enthalten. Wenn nein, Abbruch
    If Len(EAN8.s) <> 8
      ProcedureReturn "12"
    EndIf
    ;Prüfen ob wirklich numerisch
    If UCase(EAN8.s) <> LCase(EAN8.s)
      ProcedureReturn "13"
    EndIf
    
    ;Sequence erzeugen
    EAN8_Sequence.s + EAN8_Start.s                    ;Start Sequence
    For i = 0 To 7 Step 2                             ;Part 1 Sequence
      part.s = PeekS(@EAN8.s + i, 1, #PB_Unicode)
      For a = 0 To 9
        If part.s = Str(a)
          EAN8_Sequence.s + EAN8_Sequence_part1(a)
        EndIf
      Next a
    Next i
    EAN8_Sequence.s + EAN8_Center.s                   ;Center Sequence
    For i = 0 To 7 Step 2                             ;Part 2 Sequence
      part.s = PeekS(@EAN8.s + i + 8, 1, #PB_Unicode)
      For a = 0 To 9
        If part.s = Str(a)
          EAN8_Sequence.s + EAN8_Sequence_part2(a)
        EndIf
      Next a
    Next i
    EAN8_Sequence.s + EAN8_Stop.s                    ;Stop Sequence
    
    ;Prüfen ob 67 Zeichen/Linien generiert wurden
    If Len(EAN8_Sequence.s) <> 67
      ProcedureReturn "14"
    EndIf
    ProcedureReturn EAN8_Sequence.s
  EndProcedure
  
  Procedure.s Generate_EAN13_Sequence(EAN13.s)
    Protected.b i, a, b
    Protected.s part, digit1, digit1_sequence
    Protected Row
    Protected Dim EAN13_digit1.s(9)
    EAN13_digit1(0) = "000000"
    EAN13_digit1(1) = "001011"
    EAN13_digit1(2) = "001101"
    EAN13_digit1(3) = "001110"
    EAN13_digit1(4) = "010011"
    EAN13_digit1(5) = "011001"
    EAN13_digit1(6) = "011100"
    EAN13_digit1(7) = "010101"
    EAN13_digit1(8) = "010110"
    EAN13_digit1(9) = "011010"
    digit1.s = Left(EAN13.s,1)
    digit1_sequence.s = EAN13_digit1(Val(digit1.s))
    ;Wird die gesamte EAN13 Sequence enthalten
    Protected EAN13_Sequence.s  
    ;EAN13 Start, Center und Stop Sequencen
    Protected EAN13_Start.s   = "101"
    Protected EAN13_Center.s  = "01010" 
    Protected EAN13_Stop.s    = "101"   
    ;EAN13 Sequenzen part #1
    Protected Dim EAN13_Sequence_part1.s(1,9)
    EAN13_Sequence_part1(0,0) = "0001101"
    EAN13_Sequence_part1(0,1) = "0011001"
    EAN13_Sequence_part1(0,2) = "0010011"
    EAN13_Sequence_part1(0,3) = "0111101"
    EAN13_Sequence_part1(0,4) = "0100011"
    EAN13_Sequence_part1(0,5) = "0110001"
    EAN13_Sequence_part1(0,6) = "0101111"
    EAN13_Sequence_part1(0,7) = "0111011"
    EAN13_Sequence_part1(0,8) = "0110111"
    EAN13_Sequence_part1(0,9) = "0001011"
    EAN13_Sequence_part1(1,0) = "0100111"
    EAN13_Sequence_part1(1,1) = "0110011"
    EAN13_Sequence_part1(1,2) = "0011011"
    EAN13_Sequence_part1(1,3) = "0100001"
    EAN13_Sequence_part1(1,4) = "0011101"
    EAN13_Sequence_part1(1,5) = "0111001"
    EAN13_Sequence_part1(1,6) = "0000101"
    EAN13_Sequence_part1(1,7) = "0010001"
    EAN13_Sequence_part1(1,8) = "0001001"
    EAN13_Sequence_part1(1,9) = "0010111"
    ;EAN13 Sequenzen part #2
    Protected Dim EAN13_Sequence_part2.s(9)
    EAN13_Sequence_part2(0) = "1110010"
    EAN13_Sequence_part2(1) = "1100110"
    EAN13_Sequence_part2(2) = "1101100"
    EAN13_Sequence_part2(3) = "1000010"
    EAN13_Sequence_part2(4) = "1011100"
    EAN13_Sequence_part2(5) = "1001110"
    EAN13_Sequence_part2(6) = "1010000"
    EAN13_Sequence_part2(7) = "1000100"
    EAN13_Sequence_part2(8) = "1001000"
    EAN13_Sequence_part2(9) = "1110100"
    
    ; Der String muss 13 Ziffern enthalten. Wenn nein, Abbruch
    If Len(EAN13.s) <> 13
      ProcedureReturn "12"
    EndIf
    ;Prüfen ob wirklich numerisch
    If UCase(EAN13.s) <> LCase(EAN13.s)
      ProcedureReturn "13"
    EndIf
    
    ;Sequence erzeugen
    EAN13_Sequence.s + EAN13_Start.s                   ;Start Sequence
    For i = 2 To 13 Step 2                             ;Part 1 Sequence
      part.s = PeekS(@EAN13.s + i, 1, #PB_Unicode)
      Row = Val(PeekS(@digit1_sequence.s + b, 1, #PB_Unicode))
      b = b +2
      For a = 0 To 9
        If part.s = Str(a)
          EAN13_Sequence.s + EAN13_Sequence_part1(Row,a)
        EndIf
      Next a
    Next i
    EAN13_Sequence.s + EAN13_Center.s                   ;Center Sequence
    For i = 0 To 11 Step 2                              ;Part 2 Sequence
      part.s = PeekS(@EAN13.s + i + 14, 1, #PB_Unicode)
      For a = 0 To 9
        If part.s = Str(a)
          EAN13_Sequence.s + EAN13_Sequence_part2(a)
        EndIf
      Next a
    Next i
    EAN13_Sequence.s + EAN13_Stop.s                    ;Stop Sequence
    
    ;Prüfen ob 95 Zeichen/Linien generiert wurden
    If Len(EAN13_Sequence.s) <> 95
      ProcedureReturn "14"
    EndIf
    ProcedureReturn EAN13_Sequence.s
  EndProcedure
  
  Procedure Print_Barcode(List Barcode.Barcode_Parameter(), *Printer.Print_Parameter, List Text.Text_Line())  
    Protected.d Line_Thickness, Offset, EAN_Text_width, EAN_Text_height, EAN_Text_posx, Text_width, Text_height
    Protected.i a, VektorID, pages, Font_Height, EAN_FontID, EAN_Font, Init_Printer, Text_Font, Text_FontID, Lines, spaces
    Protected.s EAN_Sequence, EAN_Text, Misc_Text, test
    
    ;Drucker initialisieren
    If *Printer\Print_Requester = #False
      Init_Printer.i = DefaultPrinter()
    Else
      Init_Printer.i = PrintRequester()
    EndIf
    If Init_Printer = 0
      ProcedureReturn 15
    EndIf
    
    
    ; Drucken starten
    VektorID = PrinterVectorOutput(#PB_Unit_Millimeter)
    Init_Printer.i = StartPrinting("EAN")
    If Init_Printer.i = 0
      ProcedureReturn 15
    EndIf
    StartVectorDrawing(VektorID)
    
    For pages.i = 1 To *Printer\Pages
      ForEach Barcode()
        ;********************************************************************************************************************
        ; EAN8/13 Drucken                                                                                                      *
        ;********************************************************************************************************************
        If Barcode()\Type = "EAN8" Or Barcode()\Type = "EAN13"
          ;EAN Liniendicke ermitteln
          If Barcode()\Type = "EAN8"
            Lines.i = 67
          Else
            Lines.i = 95
          EndIf
          Line_Thickness.d = Barcode()\Width / Lines.i  
          ;Zeichensatz für EAN Ziffern laden
          If Barcode()\Font <> ""
            Font_Height.i = Round(Barcode()\Width /4, #PB_Round_Nearest)
            EAN_Font.i = LoadFont(#PB_Any, Barcode()\Font, Font_Height.i)
            If EAN_Font.i = 0
              ProcedureReturn 16
            EndIf
            EAN_FontID.i = FontID(EAN_Font)
          EndIf      
          ;Zwecks besserer Optik Leerzeichen in den EAN Text einfügen
          If Barcode()\Type = "EAN8"
            spaces.i = 14
          Else
            spaces.i = 24
          EndIf
          Misc_Text.s = Barcode()\Barcode
          For a = 0 To spaces.i Step 2
            EAN_Text.s + PeekS(@Misc_Text.s + a, 1, #PB_Unicode)
            EAN_Text.s + " "
          Next a    
          ;EAN Text Zeichnen
          If Barcode()\Font <> "" 
            VectorFont(EAN_FontID.i)
            EAN_Text_width.d = VectorTextWidth(EAN_Text.s)
            EAN_Text_height.d = VectorTextHeight(EAN_Text.s)     
            EAN_Text_posx.d = (Barcode()\Width - EAN_Text_width.d) / 2 + Barcode()\PosX
            MovePathCursor(EAN_Text_posx.d, Barcode()\PosY + Barcode()\Height - EAN_Text_height.d)
            VectorSourceColor(RGBA(Val(StringField(Barcode()\ColorDigits_RGB, 1, ",")), Val(StringField(Barcode()\ColorDigits_RGB, 2, ",")), Val(StringField(Barcode()\ColorDigits_RGB, 3, ",")), 255))
            DrawVectorText(EAN_Text.s)
            FreeFont(EAN_Font.i)
          EndIf
          ;EAN Linien Zeichnen
          EAN_Sequence.s = Barcode()\Sequence
          Offset.d = 0
          For a = 0 To Lines.i * 2 Step 2
            If PeekS(@EAN_Sequence.s + a, 1, #PB_Unicode) = "1"
              MovePathCursor(Barcode()\PosX + Offset, Barcode()\PosY)
              If Barcode()\Font <> ""
                AddPathLine(Barcode()\PosX + Offset, Barcode()\PosY + Barcode()\Height - EAN_Text_height.d)
              Else
                AddPathLine(Barcode()\PosX + Offset, Barcode()\PosY + Barcode()\Height)
              EndIf
              VectorSourceColor(RGBA(Val(StringField(Barcode()\Color1_RGB, 1, ",")), Val(StringField(Barcode()\Color1_RGB, 2, ",")), Val(StringField(Barcode()\Color1_RGB, 3, ",")), 255))
              StrokePath(Line_Thickness.d)
              Offset.d + Line_Thickness.d
            EndIf
            If PeekS(@EAN_Sequence.s + a, 1, #PB_Unicode) = "0"
              MovePathCursor(Barcode()\PosX + Offset, Barcode()\PosY)
              If Barcode()\Font <> ""
                AddPathLine(Barcode()\PosX + Offset, Barcode()\PosY + Barcode()\Height - EAN_Text_height.d)
              Else
                AddPathLine(Barcode()\PosX + Offset, Barcode()\PosY + Barcode()\Height)
              EndIf            
              VectorSourceColor(RGBA(Val(StringField(Barcode()\Color0_RGB, 1, ",")), Val(StringField(Barcode()\Color0_RGB, 2, ",")), Val(StringField(Barcode()\Color0_RGB, 3, ",")), 255))
              StrokePath(Line_Thickness.d)
              Offset.d + Line_Thickness.d
              ;Prüfen ob der Barcode im Druckbereich liegt.
              If Barcode()\PosX < *Printer\Left_edge
                ProcedureReturn 22
              ElseIf Barcode()\PosX + Barcode()\Width > *Printer\Page_width - *Printer\Right_edge
                ProcedureReturn 23
              ElseIf Barcode()\PosY < *Printer\Top_edge
                ProcedureReturn 24
              ElseIf Barcode()\PosY + Barcode()\Height > *Printer\Page_height - *Printer\Upper_edge
                ProcedureReturn 25
              EndIf
            EndIf
          Next a
          EAN_Text.s =""
          EAN_Text_width.d = 0
          EAN_Text_height.d = 0
          
        EndIf
        ;********************************************************************************************************************
        ; EAN Drucken Ende                                                                                                 *
        ;********************************************************************************************************************
      Next 
      ;********************************************************************************************************************
      ; Texte drucken                                                                                                     *
      ;********************************************************************************************************************
      ForEach Text()
        ;Zeichensatz für Text_Line laden und Text zeichnen
        If Text()\Font <> ""
          Text_Font.i = LoadFont(#PB_Any, Text()\Font, Text()\Font_Size, Text()\Text_Style)
          If Text_Font.i = 0
            ProcedureReturn 17
          EndIf
          Text_FontID.i = FontID(Text_Font.i)
        EndIf
        VectorFont(Text_FontID.i)
        Text_width.d = VectorTextWidth(Text()\Text)
        Text_height.d = VectorTextHeight(Text()\Text)     
        MovePathCursor(Text()\Text_PosX , Text()\Text_PosY) 
        VectorSourceColor(RGBA(Val(StringField(Text()\TextColor_RGB, 1, ",")), Val(StringField(Text()\TextColor_RGB, 2, ",")), Val(StringField(Text()\TextColor_RGB, 3, ",")), 255))
        DrawVectorText(Text()\Text)
        FreeFont(#PB_All)
        ;Prüfen ob der Text im Druckbereich liegt.
        If Text()\Text_PosX < *Printer\Left_edge
          ProcedureReturn 18
        ElseIf Text()\Text_PosX + Text_width.d > *Printer\Page_width - *Printer\Right_edge
          ProcedureReturn 19
        ElseIf Text()\Text_PosY < *Printer\Top_edge
          ProcedureReturn 20
        ElseIf Text()\Text_PosY + Text_height.d > *Printer\Page_height - *Printer\Upper_edge
          ProcedureReturn 21
        EndIf
      Next
      ;********************************************************************************************************************
      ; Texte drucken Ende                                                                                                *
      ;********************************************************************************************************************
      
      If pages.i < *Printer\Pages
        NewPrinterPage()
      EndIf      
    Next pages.i
    StopVectorDrawing()
    StopPrinting()
    ProcedureReturn #True
  EndProcedure
  
EndModule
; IDE Options = PureBasic 5.61 (Windows - x64)
; CursorPosition = 19
; Folding = A5
; EnableXP